@@ -203,19 +203,15 @@ __lxcfs_fuse_ops int proc_release(const char *path, struct fuse_file_info *fi)
 static uint64_t get_memlimit(const char *cgroup, bool swap)
 {
 	__do_free char *memlimit_str = NULL;
-	uint64_t memlimit = -1;
-	char *ptr;
+	uint64_t memlimit = 0;
 	int ret;
 
 	if (swap)
 		ret = cgroup_ops->get_memory_swap_max(cgroup_ops, cgroup, &memlimit_str);
 	else
 		ret = cgroup_ops->get_memory_max(cgroup_ops, cgroup, &memlimit_str);
-	if (ret > 0) {
-		memlimit = strtoull(memlimit_str, &ptr, 10);
-		if (ptr == memlimit_str)
-			memlimit = -1;
-	}
+	if (ret > 0 && safe_uint64(memlimit_str, &memlimit, 10) < 0)
+		lxcfs_error("Failed to convert memlimit %s", memlimit_str);
 
 	return memlimit;
 }
@@ -223,23 +219,20 @@ static uint64_t get_memlimit(const char *cgroup, bool swap)
 static uint64_t get_min_memlimit(const char *cgroup, bool swap)
 {
 	__do_free char *copy = NULL;
-	uint64_t memlimit = 0;
-	uint64_t retlimit;
+	uint64_t memlimit = 0, retlimit = 0;
 
 	copy = strdup(cgroup);
 	if (!copy)
 		return log_error_errno(0, ENOMEM, "Failed to allocate memory");
 
 	retlimit = get_memlimit(copy, swap);
-	if (retlimit == -1)
-		retlimit = 0;
 
 	while (strcmp(copy, "/") != 0) {
 		char *it = copy;
 
 		it = dirname(it);
 		memlimit = get_memlimit(it, swap);
-		if (memlimit != -1 && memlimit < retlimit)
+		if (memlimit > 0 && memlimit < retlimit)
 			retlimit = memlimit;
 	};
 
@@ -296,14 +289,18 @@ static int proc_swaps_read(char *buf, size_t size, off_t offset,
 	if (ret < 0)
 		return 0;
 
-	memusage = strtoull(memusage_str, NULL, 10);
+	if (safe_uint64(memusage_str, &memusage, 10) < 0)
+		lxcfs_error("Failed to convert memusage %s", memusage_str);
 
 	ret = cgroup_ops->get_memory_swap_max(cgroup_ops, cg, &memswlimit_str);
 	if (ret >= 0)
 		ret = cgroup_ops->get_memory_swap_current(cgroup_ops, cg, &memswusage_str);
 	if (ret >= 0) {
 		memswlimit = get_min_memlimit(cg, true);
-		memswusage = strtoull(memswusage_str, NULL, 10);
+
+		if (safe_uint64(memswusage_str, &memswusage, 10) < 0)
+			lxcfs_error("Failed to convert memswusage %s", memswusage_str);
+
 		swap_total = (memswlimit - memlimit) / 1024;
 		swap_free = (memswusage - memusage) / 1024;
 	}
@@ -524,7 +521,8 @@ static inline void iwashere(void)
 }
 #endif
 
-/* This function retrieves the busy time of a group of tasks by looking at
+/*
+ * This function retrieves the busy time of a group of tasks by looking at
  * cpuacct.usage. Unfortunately, this only makes sense when the container has
  * been given it's own cpuacct cgroup. If not, this function will take the busy
  * time of all other taks that do not actually belong to the container into
@@ -545,10 +543,13 @@ static double get_reaper_busy(pid_t task)
 	if (!cgroup)
 		return 0;
 	prune_init_slice(cgroup);
+
 	if (!cgroup_ops->get(cgroup_ops, "cpuacct", cgroup, "cpuacct.usage", &usage_str))
 		return 0;
 
-	usage = strtoull(usage_str, NULL, 10);
+	if (safe_uint64(usage_str, &usage, 10) < 0)
+		lxcfs_error("Failed to convert usage %s", usage_str);
+
 	return ((double)usage / 1000000000);
 }
 
@@ -558,38 +559,21 @@ static uint64_t get_reaper_start_time(pid_t pid)
 	__do_fclose FILE *f = NULL;
 	int ret;
 	uint64_t starttime;
-	/* strlen("/proc/") = 6
-	 * +
-	 * LXCFS_NUMSTRLEN64
-	 * +
-	 * strlen("/stat") = 5
-	 * +
-	 * \0 = 1
-	 * */
-#define __PROC_PID_STAT_LEN (6 + LXCFS_NUMSTRLEN64 + 5 + 1)
-	char path[__PROC_PID_STAT_LEN];
+	char path[STRLITERALLEN("/proc/") + LXCFS_NUMSTRLEN64 +
+		  STRLITERALLEN("/stat") + 1];
 	pid_t qpid;
 
 	qpid = lookup_initpid_in_store(pid);
-	if (qpid <= 0) {
-		/* Caller can check for EINVAL on 0. */
-		errno = EINVAL;
-		return 0;
-	}
+	if (qpid <= 0)
+		return ret_errno(EINVAL);
 
-	ret = snprintf(path, __PROC_PID_STAT_LEN, "/proc/%d/stat", qpid);
-	if (ret < 0 || ret >= __PROC_PID_STAT_LEN) {
-		/* Caller can check for EINVAL on 0. */
-		errno = EINVAL;
-		return 0;
-	}
+	ret = snprintf(path, sizeof(path), "/proc/%d/stat", qpid);
+	if (ret < 0 || (size_t)ret >= sizeof(path))
+		return ret_errno(EINVAL);
 
 	f = fopen_cached(path, "re", &fopen_cache);
-	if (!f) {
-		/* Caller can check for EINVAL on 0. */
-		errno = EINVAL;
-		return 0;
-	}
+	if (!f)
+		return ret_errno(EINVAL);
 
 	/* Note that the *scanf() argument supression requires that length
 	 * modifiers such as "l" are omitted. Otherwise some compilers will yell
@@ -620,7 +604,7 @@ static uint64_t get_reaper_start_time(pid_t pid)
 			"%" PRIu64, /* (22) starttime   %llu */
 		     &starttime);
 	if (ret != 1)
-		return ret_set_errno(0, EINVAL);
+		return ret_errno(EINVAL);
 
 	return ret_set_errno(starttime, 0);
 }
@@ -632,11 +616,11 @@ static double get_reaper_start_time_in_sec(pid_t pid)
 	double res = 0;
 
 	clockticks = get_reaper_start_time(pid);
-	if (clockticks == 0 && errno == EINVAL)
+	if (clockticks <= 0)
 		return log_debug(0, "Failed to retrieve start time of pid %d", pid);
 
 	ret = sysconf(_SC_CLK_TCK);
-	if (ret < 0 && errno == EINVAL)
+	if (ret < 0)
 		return log_debug(0, "Failed to determine number of clock ticks in a second");
 
 	ticks_per_sec = (uint64_t)ret;
@@ -649,7 +633,8 @@ static double get_reaper_age(pid_t pid)
 	uint64_t uptime_ms;
 	double procstart, procage;
 
-	/* We need to substract the time the process has started since system
+	/*
+	 * We need to substract the time the process has started since system
 	 * boot minus the time when the system has started to get the actual
 	 * reaper age.
 	 */
@@ -663,11 +648,6 @@ static double get_reaper_age(pid_t pid)
 		if (ret < 0)
 			return 0;
 
-		/* We could make this more precise here by using the tv_nsec
-		 * field in the timespec struct and convert it to milliseconds
-		 * and then create a double for the seconds and milliseconds but
-		 * that seems more work than it is worth.
-		 */
 		uptime_ms = (spec.tv_sec * 1000) + (spec.tv_nsec * 1e-6);
 		procage = (uptime_ms - (procstart * 1000)) / 1000;
 	}
@@ -721,7 +701,7 @@ static int proc_uptime_read(char *buf, size_t size, off_t offset,
 
 	total_len = snprintf(d->buf, d->buflen, "%.2lf %.2lf\n", reaperage, idletime);
 	if (total_len < 0 || total_len >= d->buflen)
-		return log_error(0, "Failed to write to cache");
+		return read_file_fuse("/proc/uptime", buf, size, d);
 
 	d->size = (int)total_len;
 	d->cached = 1;
@@ -730,6 +710,7 @@ static int proc_uptime_read(char *buf, size_t size, off_t offset,
 		total_len = size;
 
 	memcpy(buf, d->buf, total_len);
+
 	return total_len;
 }
 
@@ -1075,10 +1056,10 @@ static int proc_meminfo_read(char *buf, size_t size, off_t offset,
 
 	ret = cgroup_ops->get_memory_current(cgroup_ops, cgroup, &memusage_str);
 	if (ret < 0)
-		return 0;
+		return read_file_fuse("/proc/meminfo", buf, size, d);
 
 	if (!cgroup_parse_memory_stat(cgroup, &mstat))
-		return 0;
+		return read_file_fuse("/proc/meminfo", buf, size, d);
 
 	/*
 	 * Following values are allowed to fail, because swapaccount might be
@@ -1089,18 +1070,21 @@ static int proc_meminfo_read(char *buf, size_t size, off_t offset,
 		ret = cgroup_ops->get_memory_swap_current(cgroup_ops, cgroup, &memswusage_str);
 	if (ret >= 0) {
 		memswlimit = get_min_memlimit(cgroup, true);
-		memswusage = strtoull(memswusage_str, NULL, 10);
 		memswlimit = memswlimit / 1024;
+
+		if (safe_uint64(memswusage_str, &memswusage, 10) < 0)
+			lxcfs_error("Failed to convert memswusage %s", memswusage_str);
 		memswusage = memswusage / 1024;
 	}
 
-	memusage = strtoull(memusage_str, NULL, 10);
+	if (safe_uint64(memusage_str, &memusage, 10) < 0)
+		lxcfs_error("Failed to convert memusage %s", memusage_str);
 	memlimit /= 1024;
 	memusage /= 1024;
 
 	f = fopen_cached("/proc/meminfo", "re", &fopen_cache);
 	if (!f)
-		return 0;
+		return read_file_fuse("/proc/meminfo", buf, size, d);
 
 	while (getline(&line, &linelen, f) != -1) {
 		ssize_t l;
@@ -1119,9 +1103,16 @@ static int proc_meminfo_read(char *buf, size_t size, off_t offset,
 		} else if (startswith(line, "MemAvailable:")) {
 			snprintf(lbuf, 100, "MemAvailable:   %8" PRIu64 " kB\n", memlimit - memusage + mstat.total_cache / 1024);
 			printme = lbuf;
-		} else if (startswith(line, "SwapTotal:") && memswlimit > 0 &&
-			   opts && opts->swap_off == false) {
-			memswlimit -= memlimit;
+		} else if (startswith(line, "SwapTotal:") && memswlimit > 0 && opts && opts->swap_off == false) {
+			uint64_t hostswtotal = 0;
+
+			sscanf(line + STRLITERALLEN("SwapTotal:"), "%" PRIu64, &hostswtotal);
+			if (hostswtotal < memswlimit)
+				memswlimit = hostswtotal;
+
+			if (memswlimit >= memlimit)
+				memswlimit -= memlimit;
+
 			snprintf(lbuf, 100, "SwapTotal:      %8" PRIu64 " kB\n", memswlimit);
 			printme = lbuf;
 		} else if (startswith(line, "SwapTotal:") && opts && opts->swap_off == true) {
